/*******************************************************************************
 * @file    main.h
 * @author  Original F4 Discovery code modified by SOC Robotics, Inc. Development Team
 * @version V1.0.0
 * @date    07-November-2013
 * @brief   This file is a template for the AQ32 UAV control board.
 *
 * Code modified by SOC RObotics, Inc. to support the AQ32 UAV Control board.
 *
 * The original Discovery F4 code has been ported to the Coocox IDE V1.7.5 with USB CDC support
 * built-in.  This code can be used as a starting point for further development of new UAV
 * applications, Stepper Control applications or whatever application comes to mind.  CooCox
 * provides a nice user friendly development environment with ongoing community support.  This
 * version uses the latest ARM GNU tool chain (4.7-2013-q3) maintained by a team of ARM employees
 * and is available here:     https://launchpad.net/gcc-arm-embedded
 * CooCox is available here:  www.coocox.org
 *
 * Additional code has been added to support the MPU-6000 accelerometer/gyrocompass,
 * HMC5883 magnetometer and MS5611 barometer.
 *
 * There are two different versions of the AQ32 co-developed with Aeroquad by SOC Robotics, Inc.:
 *     Rev G - Final version supplied to Aeroquad
 *     Rev F - An earlier version without the XOR gate added to USART3
 *
 * Other than the additional gate added to USART3 the two versions are functionally equivalent.
 * For most applications the difference between the two is negligible.
 *
 * The AQ32 is an open hardware/open software platform targeting embedded UAV applications. This is
 * the same board supplied by Aeroquad with full Aeroquad software support.
 *
 * Step/Dir Output:
 *
 * 	STEPX	- PC9	- PWM 1
 * 	DIRX	- PC8	- PWM 2
 * 	STEPY	- PC7	- PWM 3
 * 	DIRY	- PC6	- PWM 4
 * 	STEPZ	- PA15	- PWM 5
 * 	DIRZ	- PB3	- PWM 6
 * 	STEPA	- PB4	- PWM 7
 * 	DIRA	- PB5	- PWM 8
 * 	STEPB	- PA0	- SRV 3
 * 	DIRB	- PA1	- SRV 2
 * 	STEPC	- PA2	- SRV 1
 * 	DIRC	- PA3	- SRV 0
 *
 * 	LIMITX	- PD12	- RF CH 1
 * 	LIMITY	- PD13	- RF CH 2
 * 	LIMITZ	- PD14	- RF CH 3
 * 	LIMITA	- PD15	- RF CH 4
 * 	LIMITB	- PE14	- RF CH 8
 * 	LIMITC	- PE13	- RF CH 7
 * 	ESTOP	- PE11	- RF CH 6
 *
 * 	Push button input in PA0 is changed to PC3 - AQ32 ADC 13
 *
 * *****************************************************************************/
/**
  ******************************************************************************
  * @file    main.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    19-September-2011
  * @brief   Main program body
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2011 STMicroelectronics</center></h2>
  ******************************************************************************
  */ 
/* Includes ------------------------------------------------------------------*/
#include "stm32f4xx_flash.h"
#include "main.h"
#include "usbd_cdc_core.h"
#include "usbd_usr.h"
#include "usbd_desc.h"
#include "usbd_cdc_vcp.h"

//Library config for this project!!!!!!!!!!!
#include "stm32f4xx_conf.h"
#include "string.h"
#include "math.h"
#include "stdarg.h"
#include <stdio.h>
#include <stdlib.h>


/** @addtogroup STM32F4-Discovery_Demo
  * @{
  */

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/

#define TESTRESULT_ADDRESS         0x080FFFFC
#define ALLTEST_PASS               0x00000000
#define ALLTEST_FAIL               0x55555555

#define  XON    0x11
#define  XOFF   0x13
#define  LF	    0x0a
#define  CR	    0x0d
#define  BS     0x08
#define  ESC	0x1b
#define  LEDON  'o'
#define  LEDOFF 'f'
#define  LEDFASTBLINK 0.10  // seconds
#define  LEDSLOWBLINK 0.30  // seconds

/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
#ifdef USB_OTG_HS_INTERNAL_DMA_ENABLED
  #if defined ( __ICCARM__ ) /*!< IAR Compiler */
    #pragma data_alignment = 4   
  #endif
#endif /* USB_OTG_HS_INTERNAL_DMA_ENABLED */
__ALIGN_BEGIN USB_OTG_CORE_HANDLE  USB_OTG_dev __ALIGN_END;

 uint32_t TimingDelay1,TimingDelay[6],TimeStart[6];
 int rxbufptr=0,rxbuflen=0;


/* Private function prototypes -----------------------------------------------*/

// The variable below indicates USB device is configured by host and ready to communicate.
// This variable is declared and set in the file, usb_usr.c
extern unsigned char gbUsbDeviceReady;

extern float sinfp(float x);
extern float cosfp(float x);
extern float tanfp(float x);
extern float tanfp2(float x);
extern float arctanfp(float x);
extern float lnfp(float x);
extern float sqrtp(float m);

/* Private functions ---------------------------------------------------------*/

// Errno return for math functions
int __errno(void)
 {
	 int i=0;

	 i += 1;
	 return i;
 }

void PutCharu(char inchar)
{
	unsigned char outstrg[2];

	outstrg[0] = inchar;
	outstrg[1] = 0;
	VCP_send_str(&outstrg);
}

// Grab a bunch of chars from USB RX buffer but trickly them out one by one
char GetCharnw(void)
{
	char inchar,inbuf[100];
	int tempptr;

	if(!rxbuflen) {
		// Get next batch of inchars
		rxbuflen = VCP_get_string(&inbuf[0]);
		// If buf empty nothing to return
		if(!rxbuflen)  return 0;
		inchar = inbuf[rxbufptr++];
		}
	else {
		if(rxbufptr==rxbuflen) {
			rxbuflen = rxbufptr = 0;
			return 0;
			}
		else inchar = inbuf[rxbufptr++];
		}
	return inchar;
}

char GetCharu(void)
{
	char inchar;

	while(!(inchar=GetCharnw())) ;
	return inchar;
}

void Putsu(char *inchar)
{
	int i;

	for(i=0; i<strlen(inchar); i++) PutChar(inchar[i]);
}

int printf(const char *fmt, ...)
{
	va_list args;
	int i,j;
	char printbuffer[250];

	va_start (args, fmt);
	i = vsnprintf(printbuffer, 250, fmt, args);
	va_end (args);

	// Print the string
	j = 0;
	while(printbuffer[j]!=0) PutCharu(printbuffer[j++]);

	return i;
}

int scanfu(const char *fmt, ...)
{
	char inchar;
	va_list args;
	int i,ptr;
	char scanbuffer[120];

	//printf("\n\rFormat: %s  args=%d",fmt,args);

	ptr = 0;
	while((inchar=GetCharu())!=CR) {
        scanbuffer[ptr++] = inchar;
		//putchar(inchar);
		}
	scanbuffer[ptr] = 0;

	va_start (args, fmt);
	i = vsscanf(scanbuffer, fmt, args);
	va_end (args);

	return i;
}

/**
  * @brief  Inserts a delay time.
  * @param  nTime: specifies the delay time length, in 10 ms.
  * @retval None
  */
void Delay(uint32_t nTime)
{
  TimingDelay1 = nTime;

  while(TimingDelay1 != 0);
}

void StartTimer(int timer,unsigned long delay)
{
	TimingDelay[timer] = delay;
	TimeStart[timer] = delay;
}

float StopTimer(int timer)
{
	float elapsedtime;

	elapsedtime = ((float)(TimeStart[timer] - TimingDelay[timer]))/25.0;
	TimeStart[timer] = TimingDelay[timer] = 0;
	return elapsedtime;
}
/**
  * @brief  Decrements the TimingDelay variable.
  * @param  None
  * @retval None
  */
void TimingDelay_Decrement(void)
{
	if(TimingDelay1 != 0) TimingDelay1--;
	if(TimingDelay[0] != 0) TimingDelay[0]--;
	if(TimingDelay[1] != 0) TimingDelay[1]--;
	if(TimingDelay[2] != 0) TimingDelay[2]--;
	if(TimingDelay[3] != 0) TimingDelay[3]--;
	if(TimingDelay[4] != 0) TimingDelay[4]--;
	if(TimingDelay[5] != 0) TimingDelay[5]--;
}

void Grn_Led(char state)
{
	if(state==LEDON) AQ32_LEDOn(LED4);
	else AQ32_LEDOff(LED4);
}

void Red_Led(char state)
{
	if(state==LEDON) AQ32_LEDOn(LED3);
	else AQ32_LEDOff(LED3);
}

void AQ32_Motor_Init(void)
{
	AQ32_Motor_Step_Init(STEPX);
	AQ32_Motor_Step_Init(STEPY);
	AQ32_Motor_Step_Init(STEPZ);
	AQ32_Motor_Step_Init(STEPA);
	AQ32_Motor_Step_Init(STEPB);
	AQ32_Motor_Step_Init(STEPC);
	AQ32_Motor_Dir_Init(DIRX);
	AQ32_Motor_Dir_Init(DIRY);
	AQ32_Motor_Dir_Init(DIRZ);
	AQ32_Motor_Dir_Init(DIRA);
	AQ32_Motor_Dir_Init(DIRB);
	AQ32_Motor_Dir_Init(DIRC);
}

void AQ32_Limit_Switch_Init(void)
{
	AQ32_Limit_Init(LIMITX);
	AQ32_Limit_Init(LIMITY);
	AQ32_Limit_Init(LIMITZ);
	AQ32_Limit_Init(LIMITA);
	AQ32_Limit_Init(LIMITB);
	AQ32_Limit_Init(LIMITC);
	AQ32_Limit_Init(ESTOP);
}

/**
  * @brief  Main program.
  * @param  None
  * @retval None
  */
int main(void)
{
	char redledstate,grnledstate;
	int i = 0;
    uint8_t buf[255],outstrg[100],inchar;
    uint8_t len;
    int cs;
    volatile float a,b,elapsedtime,angle,radius,angleinc;

    RCC_ClocksTypeDef RCC_Clocks;

	// Initialize System and Setup clocks.
	SystemInit();
	SystemCoreClockUpdate();

	/* SysTick end of count event each 40ms */
	RCC_GetClocksFreq(&RCC_Clocks);
	//SysTick_Config(RCC_Clocks.HCLK_Frequency /100);

	// Initialize FPU
	*((volatile unsigned long*)0xE000ED88) = 0xF << 20;

	// Initialize LEDs and User_Button on STM32F4-Discovery
    AQ32_LEDInit(LED4);
    AQ32_LEDInit(LED3);
    //STM_EVAL_LEDInit(LED5);
    //STM_EVAL_LEDInit(LED6);

    // Configure AQ32 Motor/Limit pins
    AQ32_Motor_Init();
    AQ32_Limit_Switch_Init();

	// USB Device Initialize
    USBD_Init(&USB_OTG_dev,
            USB_OTG_FS_CORE_ID, 
            &USR_desc, 
            &USBD_CDC_cb, 
            &USR_cb);

    // Wait for USB connection goes live
    while (!gbUsbDeviceReady);

    // Clean USB RX buffer
    while(VCP_get_string(&buf[0]) == 0);

    // Output signon
    printf("\n\r\n\rUSB serial DEMO\n\r");
    printf("test\n\r>");
    grnledstate = redledstate = LEDON;

    //  Main loop
    while(1) {

    	inchar = GetCharnw();

    	if(inchar) {
    		switch (inchar){
    			case 's':
    				printf("\n\rF4 Discovery Test V0.55\n\r>");
    				break;
    			case 't':
    				printf("\n\rDo 10000 circular interpolation calculations\n\r");
    				TimingDelay[4] = 10000;
    				angle = 0.125;
    				radius = 2.56;
    				angleinc = 0.0001;
    				for(i=0; i<100000; i++) {
    					a = radius * sinf(angle);
    					b = radius * cosf(angle);
    					angle += angleinc;
    					}
    				elapsedtime = ((float)(10000 - TimingDelay[4]))/25.0;
    				printf("timing delay=%d\n\r",TimingDelay[4]);
    				printf("Single precision finished in %f seconds or %f usec/loop\n\r",elapsedtime,elapsedtime*10.0);
    				TimingDelay[4] = 10000;
    				angle = 0.125;
    				radius = 2.56;
    				angleinc = 0.0001;
    				for(i=0; i<100000; i++) {
    					a = radius * sinfp(angle);
    					b = radius * cosfp(angle);
    					angle += angleinc;
    					}
    				elapsedtime = ((float)(10000 - TimingDelay[4]))/25.0;
    				printf("timing delay=%d\n\r",TimingDelay[4]);
    				printf("Single prec fp finished in %f seconds or %f usec/loop\n\r",elapsedtime,elapsedtime*10.0);
    				TimingDelay[4] = 10000;
    				angle = 0.125;
    				radius = 2.56;
    				angleinc = 0.0001;
    				printf("angle=%f radius=%f angleinc=%f\n\r",angle,radius,angleinc);
    				for(i=0; i<100000; i++) {
    					a = radius * sin(angle);
    					b = radius * cos(angle);
    					angle += angleinc;
    					}
    				printf("timing delay=%d\n\r",TimingDelay[4]);
    				elapsedtime = ((float)(10000 - TimingDelay[4]))/25.0;
    				printf("Double precision finished in %f seconds or %f usec/loop\n\r>",elapsedtime,elapsedtime*10.0);
    				break;
				case 'f':
					printf("f\n\rTry float output: 1.234\n\r");
					a = 1.234;
					printf("a = %f\n\r",a);
					i = 35;
					printf("i = %d\n\r",i);
					a = 35.45;
					printf("a = %f\n\r",a);
					printf("a = %f\n\r",12.345);
					printf("a = %f\n\r",-12.345);
					printf("i = %d\n\r",i);
					break;
				case 'd':
					printf("d\n\rRCC_Clocks.HCLK_Frequency=%ld",RCC_Clocks.HCLK_Frequency);
					printf("\n\rDelay 2 second\n\r");
					Delay(200);
					printf("finished\n\r>");
					break;
				case 'e':
					printf("d\n\rExercise All Motor step-dir outputs - read limits\n\r");
					Red_Led(LEDON);
					Grn_Led(LEDON);
					while((inchar = GetCharnw())!='e') {
						AQ32_Motor_Step_High(STEPX);
						AQ32_Motor_Step_Low(STEPX);
						AQ32_Motor_Step_High(STEPY);
						AQ32_Motor_Step_Low(STEPY);
						Delay(2);
						printf("LimitX = %d  \r",AQ32_Limit_GetState(LIMITX));
						}
					printf("\n\r>");
					Red_Led(LEDOFF);
					Grn_Led(LEDOFF);
					break;
				case CR:
					printf("\n\r>");
					break;
				default:
					printf("%c\n\r>",inchar);
					break;
				}
    		}

    	// Green Led toggle
        if(!TimingDelay[0]) {
        	if(grnledstate) {
        		Grn_Led(LEDON);
        		StartTimer(0,2);
        		grnledstate = 0;
        		}
        	else {
        		Grn_Led(LEDOFF);
        		StartTimer(0,2);
        		grnledstate = 1;
        		}
        	}

        if(!TimingDelay[1]) {
        	if(redledstate) {
        		Red_Led(LEDON);
        		StartTimer(1,4);
        		redledstate = 0;
        		}
        	else {
        		Red_Led(LEDOFF);
        		StartTimer(1,4);
        		redledstate = 1;
        		}
        	}
    	}
}

/**
  * @brief  This function handles the test program fail.
  * @param  None
  * @retval None
  */
void Fail_Handler(void)
{
  /* Erase last sector */ 
  FLASH_EraseSector(FLASH_Sector_11, VoltageRange_3);
  /* Write FAIL code at last word in the flash memory */
  FLASH_ProgramWord(TESTRESULT_ADDRESS, ALLTEST_FAIL);
 
  while(1)
  {
    /* Toggle Red LED */
    STM_EVAL_LEDToggle(LED5);
    Delay(5);
  }
}


#ifdef  USE_FULL_ASSERT

/**
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif

/**
  * @}
  */


/******************* (C) COPYRIGHT 2011 STMicroelectronics *****END OF FILE****/
